package net.sf.extcos.selector;

import static net.sf.extcos.util.Assert.iae;

import java.util.Set;

import net.sf.extcos.internal.ArraySet;
import net.sf.extcos.internal.EnumBasedReturning;
import net.sf.extcos.internal.Returning;
import net.sf.extcos.spi.ResourceResolver;
import net.sf.extcos.util.Assert;

public class ForwardingBuilder {
	private final Set<ResourceResolver> resourceResolvers = new ArraySet<ResourceResolver>();
	private final Set<StoreBinding> storeBindings = new ArraySet<StoreBinding>();
	private final ReturningSelector returningSelector = new ReturningSelector();
	private DirectReturning returning = new EnumBasedReturning(Returning.ALL);

	private boolean calledAndStore;
	private boolean calledReturning;
	private boolean calledUsing;

	public ForwardingBuilder using(final ResourceResolver... resourceResolvers) {
		Assert.state(firstEntry() && !calledUsing);
		Assert.notEmpty(resourceResolvers, iae());
		
		for (ResourceResolver resourceResolver : resourceResolvers) {
			this.resourceResolvers.add(resourceResolver);
		}
		
		calledUsing = true;
		return this;
	}

	public ReturningSelector andStore(final StoreBinding... bindings) {
		Assert.state(firstEntry());
		Assert.notEmpty(bindings, iae());

		for (StoreBinding storeBinding : bindings) {
			this.storeBindings.add(storeBinding);
		}

		calledAndStore = true;
		return returningSelector;
	}

	public void returning(final DirectReturning returning) {
		Assert.state(firstEntry());
		Assert.notNull(returning, iae());

		this.returning = returning;
		calledReturning = true;
	}

	public Set<ResourceResolver> getResourceResolvers() {
		return resourceResolvers;
	}
	
	public Set<StoreBinding> getStoreBindings() {
		return storeBindings;
	}

	public StoreReturning getReturning() {
		if (calledAndStore) {
			return returningSelector.getStoreReturning();
		}

		return returning;
	}

	private boolean firstEntry() {
		return !calledAndStore && !calledReturning;
	}
}